; ---purpose---
; Subtract two natural decimal numbers. Numbers are
; represented by lists of digits.

; ---args---
; A - list of digits
; B - list of digits (number to be subtracted)

; ---keywords---
; DIFFERENCE function, difference, subtract, subtraction
; arithmetics, natural

; ---see-also---
; DIGITS, IDIFFERENCE, PLUS, DIVIDE, QUOTIENT, REMAINDER, TIMES

; ---example---
; (DIFFERENCE '#100 '#45)

(DEFINE DIFFERENCE T)

:C LIB/DIGITS.L
:C LIB/APPEND.L
:C LIB/REVERSE.L
:C LIB/PRED.L
:C LIB/NORMALIZE.L

; ---code---
(DEFUN DIFFERENCE (A B)
	(LABEL

	; Compute difference of two digits with borrow
	; X Y - digits
	; CI CO - borrow (carry) in, out
	; RESULT - (difference . carry out)
	((DDIFF (LAMBDA (X Y CI CO)
		(COND ((EQ Y 0)
				(COND ((EQ CI 1) (DDIFF X 1 0 CO))	; subtract carry in
					(T (CONS X CO))))
			((NULL (PRED X))						; handle overflow
				(DDIFF 9 (PRED Y) CI 1))
			(T (DDIFF (PRED X) (PRED Y) CI CO)))))

	; Subtract two lists of decimal digits forming reverse
	; decimal numbers. The result is in normal order.
	; A B - reverse lists of digits
	; C - carry flag
	; R - result (normal order)
	(DIFF (LAMBDA (A B C R)
		(COND ((NULL A)
				(COND ((NULL B)				; A=() ; B=()
						(COND ((EQ C 0) R)	; carry not set? OK
							(T (BOTTOM 'DIFFERENCE))))	; A<B is undefined
					(T (BOTTOM 'DIFFERENCE))))
		((NULL B)							; A\=() ; B=() ; see above
			(COND ((EQ C 0)					; carry not set?
					(APPEND (REVERSE A) R))	; prepend rest of A
				(T (DIFF A '#1 0 R))))		; else subtract borrow
		(T (DIFF (CDR A) (CDR B)			; advance to next pair of digits
			(CDR (DDIFF (CAR A) (CAR B) C 0))	; compute borrow
			(CONS (CAR (DDIFF (CAR A)			; prepend result digit
					(CAR B) C 0))
				R)))))))
	
	(NORMALIZE (DIFF (REVERSE A) (REVERSE B) 0 NIL))))
