; ---purpose---
; Divide two natural decimal numbers, giving a
; result of the form (quotient remainder).
; Numbers are represented by lists of digits.

; ---args---
; A - list of digits (dividend)
; B - list of digits (divisor)

; ---keywords---
; DIVIDE function, quotient, modulus, division, divide
; arithmetics, natural

; ---see-also---
; DIGITS, IDIVIDE, QUOTIENT, REMAINDER, PLUS, DIFFERENCE, TIMES

; ---example---
; (DIVIDE '#11 '#3)

(DEFINE DIVIDE T)

:C LIB/DIGITS.L
:C LIB/PLUS.L
:C LIB/LESSP.L
:C LIB/EQUAL.L
:C LIB/DIFFERENCE.L
:C LIB/INCR.L
:C LIB/DECR.L
:C LIB/NORMALIZE.L
:C LIB/ZEROP.L
:C LIB/REVERSE.L
:C LIB/APPEND.L

; ---code---
(DEFUN DIVIDE (A B)
	(LABEL

	; Equalize the divisor by shifting it to the left
	; (multiplying it by 10) until it has the same number
	; of digits as the dividend.
	; A - list of digits (dividend)
	; B - reverse list of digits (divisor)
	; R - result of the form (new divisor . one I per shift)
	((EQL (LAMBDA (A B R)
		(COND ((NULL A)								; end of dividend?
				(CONS (REVERSE (CAR R)) (CDR R)))	; reverse the divisor
			((NULL B) 						; end of divisor?
				(EQL (CDR A) NIL			; go to next digit
					(CONS (CONS 0 (CAR R))	; prepend 0 to divisor
						(CONS 'I (CDR R)))))	; count shift
			(T (EQL (CDR A) (CDR B)			; default: copy a digit
				(CONS (CONS (CAR B) (CAR R)) (CDR R)))))))

	; Divide two lists of digits which have a quotient
	; less than 10.
	; A - dividend
	; B - divisor
	; R - result of the form (A/B*B . A/B)
	(DIV10 (LAMBDA (A B R)
		(COND ((LESSP (CAR R) A)
				(DIV10 A B (CONS (PLUS (CAR R) B) (INCR (CDR R)))))
			((EQUAL (CAR R) A) R)
			(T (CONS (DIFFERENCE (CAR R) B) (DECR (CDR R)))))))

	; X / 10
	(D10 (LAMBDA (X) (REVERSE (CDR (REVERSE X)))))

	; X * 10
	(X10 (LAMBDA (X) (APPEND X '#0)))

	; Divide two lists of digits
	; A - list of digits (dividend)
	; B - divisor of the form (EQL A B ...)
	; R - result (list of digits in normal order)
	(DIV (LAMBDA (A B R)
		(COND ((NULL (CDR B)) (CONS (NORMALIZE R) (CONS A NIL)))
			(T (LABEL ((QUOT (DIV10 A (CAR B) '(#0.#0))))
				(DIV (DIFFERENCE A (CAR QUOT))
					(CONS (D10 (CAR B)) (CDDR B))
					(PLUS (X10 R) (CDR QUOT)))))))))
	
	(COND ((ZEROP B) (BOTTOM 'DIVIDE))
		((LESSP A B) (LIST '#0 A))
		(T (DIV A (EQL A B '(() I)) '#0)))))
