; ---purpose---
; Check if two natural decimal numbers are in ascending
; order. Return *T*, if A<B and otherwise *F*. Numbers
; are represented by lists of decimal digits.

; ---args---
; A - list of digits
; B - list of digits

; ---keywords---
; LESSP function, less than, ordering, relation
; comparison, predicate, natural

; ---see-also---
; DIGITS, ILESSP, GREATERP, EQUAL

; ---example---
; (LESSP '#5 '#7)

:C LIB/LOGAND.L
:C LIB/PRED.L
:C LIB/REVERSE.L

; ---code---
(DEFUN LESSP (A B)
	(LABEL

	; Check the ordering of two digits
	; A - digit
	; B - digit
	; Result: A<B-> *T*; *F*
	((DLT (LAMBDA (A B)
		(COND ((EQ A B) *F*)
			((EQ A 0) *T*)
			((EQ B 0) *F*)
			(T (DLT (PRED A) (PRED B))))))

	; Check the inverse ordering of two digits
	; A - digit
	; B - digit
	; Result: A>B-> *T*; *F*
	(DGT (LAMBDA (A B) (DLT B A)))
	
	; Check ordering.
	; A - reverse list of digits
	; B - reverse list of digits
	; R - result: member of {*T*,*F*}
	(LTP (LAMBDA (A B R)
		(COND ((LOGAND (NULL A) (NULL B)) R)	; all compared
			((NULL A) *T*)						; length[a] < length[b]
			((NULL B) *F*)						; length[a] > length[b]
			(T (LTP (CDR A) (CDR B)
				(COND ((DLT (CAR A) (CAR B)) *T*)	; default:
					((DGT (CAR A) (CAR B)) *F*)	; compare digits
					(T R))))))))

	(LTP (REVERSE A) (REVERSE B) *F*)))
