; ---purpose---
; Generate permutations of a set.

; ---args---
; X - set

; ---keywords---
; PERMUTE, permutation, combination

; ---see-also---
; UNIQUE, MEMBER

; ---example---
; (PERMUTE '#ABC)

:C LIB/CAAR.L
;L LIB/CDDR.L
:C LIB/APPEND.L
:C LIB/REVERSE.L

; ---code---
(DEFUN PERMUTE (X)
	(LABEL

	; rotate a list by moving the first element to the tail
	((ROTATE (LAMBDA (X)
		(APPEND (CDR X) (LIST (CAR X)))))

	; distribute A over B by consing A to each member of B
	(DIST (LAMBDA (A B)
		(COND ((NULL B) NIL)
			(T (CONS (CONS A (CAR B)) (DIST A (CDR B)))))))

	; create permutations of each member L of X by
	; distributing (CAR L) over all permutations of (CDR L)
	(PERMLIST (LAMBDA (X R)
		(COND ((NULL X) R)
			(T (PERMLIST (CDR X)
				(APPEND (DIST (CAAR X) (PERM (CDAR X) (CDAR X) NIL)) R))))))

	; create a list of all possible rotations of a list
	; and pass it to PERMLIST
	(PERM (LAMBDA (X N R)
		(COND ((NULL N) (PERMLIST R NIL))
			((NULL (CDR X)) X)
			((NULL (CDDR X)) (LIST X (REVERSE X)))
			(T (PERM (ROTATE X) (CDR N) (CONS X R)))))))

	(PERM X X NIL)))
