; ---purpose---
; Substitute variables in S-expressions.
; For a description of association lists (alists)
; see ASSOC.

; ---args---
; A - alist containing the variable bindings
; X - S-expression to process

; ---keywords---
; SUBLIS function, substitution, substitute variables
; association lists

; ---see-also---
; ASSOC, SASSOC, SUBST

; ---example---
; (SUBLIS '((1.I) (2.II) (3.III)) '(1 + 2 - 3))

:C LIB/ASSOC.L
:C LIB/REVERSE.L
:C LIB/CAAR.L
;C LIB/CDAR.L
:C LIB/APPEND.L
:C LIB/LITATOM.L

; ---model---
; (DEFUN SUBLIS (A X)
;	(COND ((NULL X) NIL)
;		((ATOM X)
;			(COND ((NULL (ASSOC X A)) X) (T (CDR (ASSOC X A)))))
;		(T (CONS (SUBLIS A (CAR X)) (SUBLIS A (CDR X))))))

; ---model---
; ; TAIL-CALL (BUT WAY TOO COMPLEX) VERSION
; ; FOR A DETAILED EXPLANATION, SEE 'COPY'
; (DEFUN SUBLIS (A X)
;	(LABEL
;		((PROTECT (LAMBDA (X) (LIST X)))
;		(UNPROTECT (LAMBDA (X) (CAR X)))
;		(SLA (LAMBDA (X A)
;			(COND ((NULL A) X)
;				((EQ (CAAR A) X) (CDAR A))
;				(T (SLA X (CDR A))))))
;		(SLS (LAMBDA (X R S)
;			(COND ((ATOM X)
;					(COND ((NULL S)
;							(CONS (CADR R) (CAR R)))
;						((ATOM (CAR S))
;							(SLS 'CONS (CONS (CONS (CADR R) (CAR R))
;									(CDDR R))
;								(CDR S)))
;						(T (SLS (PROTECT (CAR (UNPROTECT S)))
;								(CONS (CONS (CADR R) (CAR R)) (CDDR R))
;								(CDR S)))))
;				((ATOM (UNPROTECT X))
;					(COND ((NULL S) (SLA (UNPROTECT X) A))
;						((ATOM (CAR S))
;							(SLS 'CONS (CONS (SLA (UNPROTECT X) A) R)
;								(CDR S)))
;						(T (SLS (CAR S) (CONS (SLA (UNPROTECT X) A) R)
;							(CDR S)))))
;				(T (SLS (PROTECT (CAR (UNPROTECT X))) R
;						(APPEND (LIST (PROTECT (CDR (UNPROTECT X))) 'CONS)
;							S)))))))
;		(SLS (PROTECT X) NIL NIL)))

; ---code---
; X MAY NOT CONTAIN ANY IMPROPER LISTS
(DEFUN SUBLIS (A X)
	(LABEL ((SLA (LAMBDA (X A)
			(COND ((NULL A) X)
				((EQ (CAAR A) X) (CDAR A))
				(T (SLA X (CDR A))))))
		(SLS (LAMBDA (X R)
			(COND ((NULL X) (REVERSE R))
				((ATOM X) (SLA X A))
				((LOGAND (LITATOM (CAR X)) (LITATOM (CDR X)))
					(CONS (SLS (CAR X) NIL) (SLS (CDR X) NIL)))
				(T (SLS (CDR X) (CONS (SLS (CAR X) NIL) R)))))))
		(SLS X NIL)))
