; ---purpose---
; Retrieve a binding from an association tree.
; An association tree (atree) is an ordered binary
; tree containing (symbol.value) pairs. Each node
; of the tree has the format
; -
; (left (symbol.value) right)
; -
; where left and right are either nodes or NIL.

; ---args---
; X - key to find
; A - atree
; P - predicate defining order of tree

; ---keywords---
; TREE-ASSOC function, association lists, association trees,
; trees, ordered trees

; ---see-also---
; TREE-INSERT, ASSOC

; ---example---
; :L LIB/LESSP.L
; (DEFINE TREE
;   '(((() (#1 1) (() (#2 2) ())) (#3 3) (() (#4 4) ())) (#5 5) ()))
; (TREE-ASSOC '#1 TREE LESSP)

:C LIB/EQUAL.L
:C LIB/CAAR.L
;C LIB/CADR.L
:C LIB/CAAAR.L
;C LIB/CADDR.L

; ---code---
(DEFUN TREE-ASSOC (X A P)
	(LABEL ((TASSOC (LAMBDA (A)
		(COND ((NULL A) NIL)
			((EQUAL (CAADR A) X) (CADR A))
			((P X (CAADR A)) (TASSOC (CAR A)))
			(T (TASSOC (CADDR A)))))))
		(TASSOC A)))

