; ---purpose---
; Convert an ordered list into a balanced tree.

; ---args---
; X - list
; P - predicate defining order

; ---keywords---
; TREE-BALANCE function, tree, balanced tree, ordered trees,
; sorting, ordering

; ---see-also---
; TREE-INSERT, TREE-SORT, TREE-FLATTEN, TREE-ASSOC

; ---example---
; :L LIB/LESSP.L
; (TREE-BALANCE '(#1 #2 #3 #4 #5 #6 #7) LESSP)

:C LIB/REVERSE.L
:C LIB/TREE-INSERT.L
:C LIB/CAAR.L
;C LIB/CADR.L
;C LIB/CDDR.L

; ---code---
(DEFUN TREE-BALANCE (X P)
	; Traverse the list using divide and conquer:
	; first insert the middle element, then the
	; left sub-list and finally the right sub-list.
	(LABEL ((DIVIDE-LIST (LAMBDA (X RX TREE)
			(COND ((NULL X) TREE)
				((EQ (CAR X) (CAR RX))
					(DIVIDE-LIST (CDR X) (REVERSE (CDR X))
						(DIVIDE-LIST (REVERSE (CDR RX)) (CDR RX)
							(TREE-INSERT (CAR X) TREE P))))
				((EQ (CAR X) (CADR RX))
					(DIVIDE-LIST (CDDR X) (REVERSE (CDDR X))
						(DIVIDE-LIST (REVERSE (CDR RX)) (CDR RX)
							(TREE-INSERT (CAR RX) TREE P))))
				(T (DIVIDE-LIST (CDR X) (CDR RX) TREE))))))
		(DIVIDE-LIST X (REVERSE X) NIL)))

