; ---purpose---
; Compute the union of two sets.
; Sets are represented by lists
; of unique members.

; ---args---
; A - list
; B - list

; ---keywords---
; UNION function, union, sets, set theory

; ---see-also---
; INTERSECTION, MEMBER, REMOVE, UNIQUE

; ---example---
; (UNION '(ALPHA BETA GAMMA) '(BETA DELTA))

:C LIB/MEMBER.L
:C LIB/APPEND.L
:C LIB/REVERSE.L
:C LIB/UNIQUE.L

; ---model---
; (DEFUN UNION (A B)
;	(COND ((NULL A) B)
;		((MEMBER (CAR A) B) (UNION (CDR A) B))
;		(T (CONS (CAR A) (UNION (CDR A) B)))))

; ---model---
; ; TAIL-RECURSIVE VERSION
; (DEFUN UNION (A B)
;	(LABEL ((UON (LAMBDA (A B R)
;		(COND ((NULL A) (APPEND (REVERSE R) B))
;			((MEMBER (CAR A) B) (UON (CDR A) B R))
;			(T (UON (CDR A) B (CONS (CAR A) R)))))))
;	(UON A B NIL)))

; ---code---
(DEFUN UNION (A B) (UNIQUE (APPEND A B)))
