; ---purpose---
; Convert a LABEL into an equivalent LAMBDA expression.
; --
; This program proves that any LABEL can be converted
; into an equivalent LAMBDA expression by automating
; this process.
; --
; Notice that
; --
; ((UNLABEL UNLABEL) UNLABEL)
; --
; breaks ArrowLISP, since it does not perform full
; beta-reduction.

; ---args---
; X - expression to convert

; ---keywords---
; LABEL, LAMBDA, scoping, example

; ---example---
; (UNLABEL '(LABEL ((A '#5) (B '#7)) (PLUS A B)))

:L LIB/REVERSE.L
:L LIB/APPEND.L
:L LIB/CAAR.L
;L LIB/CADR.L
:L LIB/CAAAR.L
;L LIB/CADDR.L
;L LIB/CADAR.L

; ---code---
(DEFUN UNLABEL(X)
	(LABEL

		; Create list of symbols to bind in LABEL
		((SYMLIST (LAMBDA (X R)
			(COND ((NULL X) (REVERSE R))
				(T (SYMLIST (CDR X) (CONS (CAAR X) R))))))

		; Create list of (unlabeled) values to bind
		(VALLIST (LAMBDA (X R)
			(COND ((NULL X) (REVERSE R))
				(T (VALLIST (CDR X) (CONS (ULBL (CADAR X)) R))))))

		; Extract (unlabeled) term of label
		(TERM (LAMBDA (X) (ULBL (CADDR X))))

		; Create lambda expression
		(MAKELAMBDA (LAMBDA (X)
			(APPEND (LIST (LIST 'LAMBDA (SYMLIST (CADR X) NIL) (TERM X)))
				(VALLIST (CADR X) NIL))))

		; Traverse expression tree, replacing labels with lambda
		(ULBL (LAMBDA (X)
			(COND ((ATOM X) X)
				((EQ (CAR X) 'QUOTE) X)
				((EQ (CAR X) 'LABEL) (MAKELAMBDA X))
				(T (CONS (ULBL (CAR X)) (ULBL (CDR X))))))))

	(ULBL X)))
